/*
  Program shows the change of temperature in a range from -15 to 35 degrees celsius
  as a color of an RGB LED, 
  the LED changes from full blue (-15) over green to red (+35)
  2 different LEDs show a frost and a heat alarm
  
  Programm zeigt die Temperatur allmählichen Temperaturwechsel von -15 bis +35 Grad
  und zeigt Temperatur in diesem Bereich mit einem Farbverlauf in einem RGB-LED an, 
  wobei kalte Temperaturen blau, mittlere grün und heisse Temperaturen rot angezeigt werden
  zusätzlich gibt es 2 LEDs für einen Frost- und einen Hitzealarm
  unterhalb einer Maximallimite brennt das blaue LED
  und oberhalb einer Minimallimite das rote LED */

int temperaturePin = 0; //the analog pin the TMP36's Vout (sense) pin is connected to
                        //the resolution is 10 mV / degree centigrade 
                        //(500 mV offset) to make negative temperatures an option
  
int ledAnalogOne[] = {3, 5, 6}; //3 PWM-LEDs: 3 red, 5 green, 6 blue
int ledBlue = 7;
int ledRed = 8;

//variables for the temperature and the temperature range                              
float temperature = 15.0;
int range = 50;
int start = -15;

int maximumLedBlue = 0; //frost alarm
int minimumLedRed = 25; //heat alarm

int red = 0;
int green = 0;
int blue = 0;
float tmp = 0.0;
int mydelay = 1000;

byte  black[] = {0, 0, 0}; 

void setup(){
  Serial.begin(9600);
  pinMode(ledBlue, OUTPUT);     
  pinMode(ledRed, OUTPUT);     
  for(int i = 0; i < 3; i++){
   pinMode(ledAnalogOne[i], OUTPUT);   //Set the three LED pins as outputs
  }
  
  setColor(ledAnalogOne, black);      
}

void loop(){
	temperature = getTemperature(temperaturePin); 
	tmp = 2 * 255 / range;

        //converts a temperature to a RGB color
	if (temperature < start) {
	  red = 0;
	  green = 0;
	  blue = 255;
	} else if (temperature <  start + (range / 2)) {
	  red = 0;
	  green = (temperature - start) * tmp;
	  blue = 255 - ((temperature - start) * tmp);
	} else if (temperature < start + range) {
	  red = (temperature - (range/2)- start) * tmp;
	  green = 255 - ((temperature - (range/2) - start) * tmp);
	  blue = 0;
	} else {
	  red = 255;
	  green = 0;
	  blue = 0;
	}

	//frost alarm
	if (temperature <= maximumLedBlue) {
          // blue LED is turned on
	  digitalWrite(ledBlue, HIGH);  
	} else {
         // blue LED is turned off
	  digitalWrite(ledBlue, LOW); 
	} 

	//heat alarm
	if (temperature >= minimumLedRed) {
         // red LED is turned on
	  digitalWrite(ledRed, HIGH);   
	} else {
         // red LED is turned off
	  digitalWrite(ledRed, LOW); 
	} 

	byte color[] = {red, green, blue};
	printToSerial(temperature, color);
	setColor(ledAnalogOne, color);
	//pauses, before the temperature is read again
	delay(mydelay);
}

//fetches a random temperature value
float getRandomTemperature(int start, int range) {
  randomSeed(millis());
  return random(start, start + range);
}

//sets the RGB LED to a RGB color
void setColor(int* led, byte* color){
  //iterates through the 3 pins
  for(int i = 0; i < 3; i++){             
   analogWrite(led[i], 255 - color[i]);                  
 }
}

//prints the temperature and RGB values at the serial interface
void printToSerial(float temperature, byte* color) {
  Serial.print("T: ");
  Serial.print(temperature);
  Serial.print("; RGB: ");
  Serial.print(color[0]);
  Serial.print("/");
  Serial.print(color[1]);
  Serial.print("/");
  Serial.println(color[2]);
}

float getTemperature(int analogPin){
  /* converting from a 0 to 1024 digital range
     to 0 to 5 volts (each 1 reading equals ~ 5 millivolts */
  float voltage = (analogRead(analogPin) * .004882814); 
                                     
  //convert into a temperature		
  return (voltage - .5) * 100;        
}
